# -*- coding: utf-8 -*-
"""
Pieter Eendebak <pieter.eendebak@gmail.com>

"""

#%%
import numpy as np
import unittest

import oapackage
import oapackage.conference

#%%


def _statistics_equal(statistics, expected):
    np.testing.assert_array_almost_equal(np.array(statistics), np.array(expected))


class TestJcharacteristics(unittest.TestCase):

    def test_Jcharacteristics(self):
        array = oapackage.exampleArray(46, 0)

        JJ = oapackage.Jcharacteristics_conference(array, number_of_columns=3)
        self.assertEqual(len(JJ), oapackage.choose(array.n_columns, 3))

        JJ = oapackage.Jcharacteristics_conference(array, 4)
        self.assertEqual(len(JJ), oapackage.choose(array.n_columns, 4))
        hist = np.histogram(np.abs(JJ), bins=np.arange(0, 21, 2))[0]
        self.assertEqual(list(hist), [9,  0, 52,  0,  8,  0,  1,  0,  0,  0])


class TestConferenceStatistics(unittest.TestCase):

    def test_momentMatrix(self):
        M1 = np.array([[1., 0., 1. / 3],
                       [0., 1. / 3, 0.],
                       [1. / 3, 0., 0.2]])
        M = oapackage.conference.momentMatrix(1)
        np.testing.assert_array_equal(M, M1)
        M = oapackage.conference.momentMatrix(4)
        self.assertEqual(M.shape, (15, 15))

    def test_conference_model_statistics(self):
        arrays = [oapackage.exampleArray(idx, 0).selectFirstColumns(4) for idx in [46, 47, 48]]
        dsds = [oapackage.conference2DSD(array) for array in arrays]
        statistics = [oapackage.conference.modelStatistics(al, verbose=2) for al in dsds]
        expected = [(0, 0, 0), (0, 0, 0), (1, 17.081221494279234, 1.7610790263409104)]
        _statistics_equal(statistics, expected)

        arrays = [oapackage.exampleArray(idx, 0).selectLastColumns(4) for idx in [46, 47, 48]]
        dsds = [oapackage.conference2DSD(array) for array in arrays]
        statistics = [oapackage.conference.modelStatistics(al, verbose=0) for al in dsds]
        expected = [(1, 17.312358463836613, 1.8538666749267378), (1, 17.312358463836613,
                                                                  1.8538666749267378), (1, 17.08122149427922, 1.7610790263409093)]
        _statistics_equal(statistics, expected)

    def test_conference_projection_statistics(self):
        arrays = [oapackage.exampleArray(idx, 0) for idx in [45, 46, 47, 48]]
        statistics = [oapackage.conference.conferenceProjectionStatistics(array, verbose=1) for array in arrays]
        expected = [(0.9857142857142858, 16.831420420862223, 1.7523727421346018), (0.9857142857142858, 16.782711360159983, 1.7515927378529395),
                    (0.9714285714285714, 16.594006884359878, 1.7298655460036123), (1.0, 16.80031370680457, 1.7750342960753236)]
        _statistics_equal(statistics, expected)

    def test_leftDivide(self):
        A = np.array([[1, 2], [3, 4]])
        B = np.array([[51, -2], [3, 4]])
        C = oapackage.conference._leftDivide(A, B)
        np.testing.assert_array_almost_equal(C, np.array([[-99.,   8.], [75.,  -5.]]))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""

Example script for Python interface to Orthogonal Array code.

mathescape, numbersep=5pt, label=Python session, gobble=0, frame=lines, framesep=1.5mm, bgcolor=bg

> pygmentize -P "bgcolor=.95,.95,.95" -P "style=colorful" -f html example_python_testing.py
> pygmentize -S colorful -f html > style.css; cp style.css ~/misc/homepage/oapage/


@author: Pieter Eendebak
"""


#import oalib
import oapackage

print('oalib version: %s' % oapackage.version() )

al=oapackage.exampleArray(0)
al.showarray()
print('D-efficiency %f, rank %d' % (al.Defficiency(), al.rank()) )
print('Generalized wordlength pattern: %s' % str(al.GWLP()))


al=oapackage.exampleArray(1)
print('Generalized wordlength pattern: %s' % str(al.GWLP()))





""" Orthogonal Array package test functions
"""

import sys
import os
import numpy as np
import tempfile
import logging
import importlib
import unittest
if sys.version_info >= (3, 4):
    import unittest.mock as mock
    from unittest.mock import patch
    python3 = True
else:
    try:
        import mock
        from mock import patch
    except ImportError as ex:
        logging.exception(ex)
        raise Exception('to perform tests with python2 install the mock package (see https://pypi.org/project/mock/)')
    python3 = False

import oalib
import oapackage
import oapackage.scanf
import oapackage.Doptim
import oapackage.graphtools


def only_python3(function):
    if python3:
        def only_python3_function(*args, **kwargs):
            return function(*args, **kwargs)
    else:
        def only_python3_function(*args, **kwargs):
            return None
    return only_python3_function


def autodoctest():
    """ Test the module using autodoc
    Example:
      >>> import oapackage
      >>> arrayclass=oapackage.arraydata_t(2, 40, 0, 7)
      >>> print(arrayclass)
      arrayclass: N 40, k 7, strength 0, s {2,2,2,2,2,2,2}, order 0

    """
    return


class TestMisc(unittest.TestCase):

    def test_reduceGraphNauty(self):
        G = np.zeros((5, 5), dtype=int)
        G[1, 0] = G[0, 1] = 1
        v = oapackage.reduceGraphNauty(G)
        assert(len(v) == G.shape[0])

    def test_exampleArray(self):
        # test a selection of the example arrays
        al = oapackage.exampleArray(5, 0)
        assert(al.md5() == '3885c883d3bee0c7546511255bb5c3ae')
        al = oapackage.exampleArray(20, 0)
        assert(np.array(al).shape == (24, 3))
        al = oapackage.exampleArray(40, 0)
        assert(np.array(al).shape == (14, 5))

    def test_scanf(self):
        r = oapackage.scanf.sscanf('1', '%d')
        assert(r[0] == 1)

    def test_oa2graph(self):
        al = oapackage.exampleArray(2, 0)
        adata = oapackage.arraylink2arraydata(al)
        g = oapackage.graphtools.oa2graph(al, adata)
        assert(g[0].shape == (34, 34))


def test_numpy_interface(verbose=0):
    A = np.eye(3, 4).astype(int)
    A[0, :] = [10, 20, 30, 50]
    A[2, :] = [-3, -4, -5, -6]

    if verbose:
        print('makearraylink')
    al = oapackage.makearraylink(A)

    np.testing.assert_array_equal(np.array(al), A)

    if verbose:
        al.showarray()

    if verbose:
        print('direct')
    al = oapackage.array_link(A)
    if verbose:
        al.showarray()
    Ax = np.array(al)
    if verbose:
        print(A)
        print(Ax)

    try:
        # not possible right now...
        if verbose:
            print('direct float')
        A = np.eye(3).astype(float)
        al = oapackage.array_link(A)
        if verbose:
            al.showarray()
    except:
        pass


def test_nauty(verbose=0):
    if verbose:
        print('test_nauty: test reduction to normal form')
    al = oapackage.exampleArray(0, verbose)
    alr = al.randomperm()
    tr = oapackage.reduceOAnauty(alr)
    alx = tr.apply(alr)
    assert(alx == al)


def miscunittest(verbose=1):
    """ Perform some unit testing, return True if succesfull """
    if verbose:
        print('oapackage: unittest: oalib version %s' % oalib.version())
    al = oalib.array_link()
    ii = 0
    al = oalib.exampleArray(ii, 0)

    arrayclass = oalib.arraydata_t(2, 16, 2, 6)  # fine
    alist = [arrayclass.create_root()]
    r = oalib.extend_arraylist(alist, arrayclass)
    if len(r) != 3:
        raise Exception('extension generation for %s failed' % (arrayclass, ))

    if not isinstance(al.getarray(), np.ndarray):
        print(
            'oapackage: unittest: error: array interface not working properly')
    else:
        if not al[2, 0] == al.getarray()[2, 0]:
            print(
                'oapackage: unittest: error: array interface not working properly')

    arrayclass = oalib.arraylink2arraydata(al)

    if verbose >= 2:
        print('unittest: calculate efficiencies')
    Deff = al.Defficiency()
    aa = oalib.Aefficiencies(al)
    assert(aa[0] == 1.0)
    assert(aa[1] == 1.0)
    assert(aa[2] == 1.0)

    if verbose >= 2:
        print('## oapackage test: example array %d: Deff %.3f' % (ii, Deff))

    # DOP reduction
    if verbose >= 2:
        print('unittest: test delete-one-factor GWLP reduction')
    al = oalib.exampleArray(5, verbose)
    al2 = al.randomperm()

    alr = al.reduceDOP()
    al2r = al2.reduceDOP()
    if not alr == al2r:
        print('error: DOP reduced arrays unequal!: %d' % (alr == al2r))
        print('alr')
        alr.showarraycompact()
        print('al2r')
        al2r.showarraycompact()
        return False

    transformation = oalib.reductionDOP(al)
    check = transformation.apply(al) == al.reduceDOP()
    if not check:
        print('error: DOP reduction transformation is invalid')

    # test graphtools
    if verbose >= 2:
        print('unittest: test graphtools')
    from oapackage.graphtools import oa2graph
    arrayclass = oalib.arraylink2arraydata(al)
    _ = oa2graph(al, arrayclass)

    test_numpy_interface()
    test_nauty()

    return True

#%%


class TestOAfiles(unittest.TestCase):
    """ Test functionality related to orthogonal array files """

    def test_misc_file_operations(self):
        array_filename = tempfile.mktemp(suffix='.oa')
        lst = [oapackage.exampleArray(4, 0)]
        oapackage.writearrayfile(array_filename, lst)
        assert(oapackage.oahelper.oaIsBinary(array_filename) is False)
        oapackage.writearrayfile(array_filename, oapackage.exampleArray(4, 0), oapackage.ABINARY)
        assert(oapackage.oahelper.oaIsBinary(array_filename))

        oapackage.oahelper.oainfo(array_filename)

        _ = oapackage.oahelper.compressOAfile(array_filename)

    def test_selectArrays(self):
        array_filename = tempfile.mktemp(suffix='.oa', dir=tempfile.tempdir)
        array_filename_out = tempfile.mktemp(suffix='.oa', dir=tempfile.tempdir)
        oapackage.writearrayfile(array_filename, [oapackage.exampleArray(4, 0), oapackage.exampleArray(4, 0)])
        oapackage.oahelper.selectArrays(array_filename, array_filename_out, [
                                        1], afmode=oalib.ABINARY, verbose=1, cache=0)
        oapackage.oahelper.selectArrays(array_filename, array_filename_out,
                                        [-1], afmode=oalib.ABINARY, verbose=1, cache=0)

    def test_nArrayFile(self):
        array_filename = tempfile.mktemp(suffix='.oa', dir=tempfile.tempdir)
        oapackage.writearrayfile(array_filename, [oapackage.exampleArray(4, 0)])
        self.assertEqual(oapackage.oahelper.nArrayFile(array_filename), 1)
        self.assertEqual(oapackage.oahelper.nArrayFile('notavalidfile.oa'), -1)

    def test_findfiles(self):
        array_filename = tempfile.mktemp(suffix='.oa', dir=tempfile.tempdir)
        lst = [oapackage.exampleArray(4, 0)]
        oapackage.writearrayfile(array_filename, lst)
        lst = oapackage.oahelper.findfiles(tempfile.tempdir, '.*oa')
        self.assertIn(os.path.split(array_filename)[-1], lst)

    def test_findfilesR(self):
        _ = oapackage.oahelper.findfilesR(tempfile.tempdir, '.*oa')

    def test_checkArrayFile(self):
        a = tempfile.mktemp(suffix='.oa')
        self.assertFalse(oapackage.oahelper.checkArrayFile(a))
        self.assertTrue(oapackage.oahelper.checkArrayFile(a, -1))

    def test_finddirectories(self):
        _ = oapackage.oahelper.finddirectories(os.getcwd())


class TestParetoFunctionality:

    def test_selectParetoArrays(self):

        arrays = [oapackage.array_link(np.array([[ii]])) for ii in range(5)]
        pareto_object = oapackage.ParetoLongLong()

        for ii in range(len(arrays)):
            value = [ii, ii % 2]
            pareto_object.addvalue(value, ii)
        pareto_object.show(2)

        selected = oapackage.oahelper.selectParetoArrays(arrays, pareto_object)
        self.assertEqual(selected, arrays[4, 5])


class TestOAhelper(unittest.TestCase):
    """ Test functionality contained in oahelper module """

    def setUp(self):
        self.test_array = oapackage.exampleArray(1, 0)

    def test_helmert_contrasts(self):
        hc = oapackage.oahelper.helmert_contrasts(2, verbose=0)
        np.testing.assert_array_almost_equal(hc, np.array([[-1.], [1.]]))

        hc = oapackage.oahelper.helmert_contrasts(3, verbose=0)
        np.testing.assert_array_almost_equal(hc, np.array([[-1.22474487, -0.70710678],
                                                           [1.22474487, -0.70710678],
                                                           [0.,  1.41421356]]))

        hc = oapackage.oahelper.helmert_contrasts(10, verbose=0)
        np.testing.assert_array_almost_equal(hc[0], np.array([-2.23606798, -1.29099445, -0.91287093, -0.70710678, -0.57735027,
                                                              -0.48795004, -0.42257713, -0.372678, -0.33333333]))

        for num_levels in [4, 10, 12]:
            hc = oapackage.oahelper.helmert_contrasts(num_levels, verbose=0)
            np.testing.assert_array_almost_equal(hc.T.dot(hc), num_levels * np.eye(num_levels - 1))

    def test_array2latex(self):

        latex_str = oapackage.oahelper.array2latex(np.array(self.test_array))
        self.assertEqual(latex_str[0:15], r'\begin{tabular}')
        latex_str = oapackage.oahelper.array2latex(np.array(self.test_array), mode='psmallmatrix')
        self.assertEqual(latex_str[0:15], r'\begin{psmallma')
        latex_str = oapackage.oahelper.array2latex(np.array(self.test_array), mode='pmatrix')
        self.assertEqual(latex_str[0:15], r'\begin{pmatrix}')

    @only_python3
    def test_gwlp2str(self):
        with self.assertWarns(UserWarning):
            self.assertEqual(oapackage.oahelper.gwlp2str([1, 2, 3]), '')
        self.assertEqual(oapackage.oahelper.gwlp2str([1, 0, .1]), '1.00,0.00,0.10')

    def test_parseProcessingTime(self):
        filename = tempfile.mktemp(suffix='.txt')
        with open(filename, 'wt') as fid:
            fid.write('# my logfile\n')
            fid.write('#time start: 2012-01-19 17:21:00\n')
            fid.write('#time end: 2012-01-19 18:21:00\n')
        dtt = oapackage.oahelper.parseProcessingTime(filename, verbose=2)
        self.assertEqual(dtt, 3600.)

    def test_argsort(self):
        idx = oapackage.oahelper.argsort([1, 2, 3])
        assert(idx == [0, 1, 2])
        idx = oapackage.oahelper.argsort([2, 2, 1])
        assert(idx == [2, 0, 1])

    @only_python3
    def test_plot2Dline(self):
        if importlib.util.find_spec('matplotlib') is not None:
            with mock.patch('matplotlib.pyplot.plot') as MockPlt:
                _ = oapackage.oahelper.plot2Dline([1, 0, 0])
                self.assertTrue(MockPlt.called)

    @only_python3
    def test_deprecated(self):
        def func():
            return 'hi'
        deprecated_function = oapackage.oahelper.deprecated(func)
        with self.assertWarns(Warning):
            _ = deprecated_function()

    def test_formatC(self):
        c_code = oapackage.oahelper.formatC(self.test_array)
        self.assertEqual(
            c_code, '\tarray_link array ( 16,5, 0 );\n\tint array_data_tmp[] = {0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,1,0,0,0,1,0,1,1,1,0,1,1,1,0,0,0,1,0,0,1,0,1,0,1,1,1,0,1,1,0,0,1,0,0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0};\tarray.setarraydata(array_data_tmp, array.n_rows * array.n_columns);\n')

    def test_runExtend(self):
        N = 24
        k = 5
        t = 2
        r = oapackage.oahelper.runExtend(N, 4, t=t, l=2, verbose=3)
        rx = oapackage.oahelper.runExtend(N, k, t=t, l=2, initsols=r, verbose=3)
        self.assertTrue(len(r) == 10)
        self.assertTrue(len(rx) == 63)

    def test_joinArrayLists(self):
        l1 = [oapackage.exampleArray(2, 0)]
        l2 = [oapackage.exampleArray(2, 0), oapackage.exampleArray(2, 0)]
        l = oapackage.oahelper.joinArrayLists([l1, l2])
        assert(len(l) == len(l1) + len(l2))

    def test_checkFiles(self):
        self.assertTrue(oapackage.oahelper.checkFiles([], -1))
        self.assertTrue(oapackage.oahelper.checkFiles(['nosuchfile'], -1))
        self.assertTrue(oapackage.oahelper.checkFiles('nosuchfile', -1))

    def test_checkFilesOA(self):
        r = oapackage.oahelper.checkFilesOA([], cache=1, verbose=0)
        self.assertTrue(r)

    def test_floatformat(self):
        s = oapackage.oahelper.floatformat(3.14, mind=2, maxd=4)
        self.assertEqual(s, '3.14')
        s = oapackage.oahelper.floatformat(3.14, mind=1, maxd=2)
        self.assertEqual(s, '3.1')

    def test_safemin_safemax(self):
        r = oapackage.oahelper.safemin(np.array([1, -2, 3]), default=0)
        self.assertEqual(r, -2)
        r = oapackage.oahelper.safemin(np.array([]), default=-2)
        self.assertEqual(r, -2)
        r = oapackage.oahelper.safemax(np.array([1, -2, 3]), default=0)
        self.assertEqual(r, 3)
        r = oapackage.oahelper.safemax([], default=-23)
        self.assertEqual(r, -23)

    def test_choose(self):
        test_cases = [((3, 2), 3), ((10, 1), 10), ((5, 2), 10), ((1, 0), 1), ((-1, 0), 1)]
        for args, expected in test_cases:
            result = oapackage.oahelper.choose(*args)
            self.assertEqual(result, expected)

    def test_create_pareto_element(self):
        values = [1, 2, 3]
        p = oapackage.oahelper.create_pareto_element(values, pareto=None)
        self.assertEqual(p, values)

        self.assertRaises(Exception, oapackage.oahelper.create_pareto_element, dict())

        pareto = oalib.ParetoMultiDoubleLong()
        pareto_element = oapackage.oahelper.create_pareto_element([[1, 2], [3, 4]], pareto)
        self.assertEqual(pareto_element.size(), 2)

        pareto = oalib.ParetoMultiLongLong()
        pareto_element = oapackage.oahelper.create_pareto_element([[1, 2], [3, 4]], pareto)
        self.assertEqual(pareto_element.size(), 2)

    def test_designStandardError(self):
        al = oapackage.exampleArray(14, 0)
        v = oapackage.oahelper.designStandardError(al)
        self.assertAlmostEqual(v[0], 0.3747931073686535)
        al = oapackage.exampleArray(9, 0)
        v = oapackage.oahelper.designStandardError(al)
        np.testing.assert_array_almost_equal(v[1], np.array([0.1679305, 0.17229075, 0.17286095, 0.17287786, 0.17303912,
                                                             0.17353519, 0.17548291]))

    def test_fac(self):
        self.assertEqual(oapackage.oahelper.fac(4), 24)

    def test_testHtml(self):
        import webbrowser
        with patch.object(webbrowser, "open", return_value=None):
            oapackage.oahelper.testHtml('<p>hi</p>')

    def test_bounds(self):
        b = oapackage.oahelper.DefficiencyBound(.8, 4, 6)
        self.assertAlmostEqual(b, 0.8944271909999)

    def test_array2html(self):
        X = np.array([[1, 2], [3, 4]], dtype='U200')
        h = oapackage.oahelper.array2html(X, header=1, tablestyle='border-collapse: collapse;',
                                          trclass='', tdstyle='', trstyle='', thstyle='')
        assert('table' in str(h))

    def test_sortrows(self):
        a = np.array([[1, 1], [-2, 2], [-3, 3], [-4, 4], [5, 5]])
        s = oapackage.oahelper.sortrows(a)
        self.assertTrue(np.all(s == [3, 2, 1, 0, 4]))

        x = -np.array([[0, 0, 1], [0, 1, 0], [0, 1, 1]])
        idx = oapackage.oahelper.sortrows(x)
        assert(np.all(idx == np.array([2, 1, 0], dtype=np.int64)))

        x = np.array([])
        idx = oapackage.oahelper.sortrows(x)
        self.assertTrue(len(idx) == 0)

    def test_sortcols(self):
        a = np.array([[1, 1], [-2, 2], [-3, 3], [-4, 4], [5, 5]]).T
        s = oapackage.oahelper.sortcols(a)
        self.assertTrue(np.all(s == [3, 2, 1, 0, 4]))



if __name__ == '__main__':
    """ Test code """
    unittest.main()



#!/usr/bin/env python

"""
setup.py file for OApackage
"""

# %% Load packages
from setuptools import setup, find_packages
from setuptools import Extension
from setuptools.command.test import test as TestCommand
from distutils.command.build import build as distutils_build
from setuptools.command.install import install as setuptools_install
import setuptools.command.build_ext 

from codecs import open  # To use a consistent encoding
from os import path
import os
import sys
import logging
import platform
import subprocess
import re

try:
    import numpy as np
except ImportError:
    raise RuntimeError(
        "numpy cannot be imported. numpy must be installed "
        "prior to installing OApackage")

npinclude = np.get_include()
setup_directory = path.abspath(path.dirname(__file__))

is_python3 = sys.version_info >= (3, 4)

# %%


def checkZlib(verbose=0):
    """ Check whether zlib is available 

    Code adapted from http://stackoverflow.com/questions/28843765/setup-py-check-if-non-python-library-dependency-exists    
    """
    ret_val = True
    try:
        import os
        import distutils.ccompiler
        import distutils.sysconfig
        import tempfile
        from distutils.errors import CompileError, LinkError

        # create test file...
        c_code = """
            #include <zlib.h>
            #include <stdio.h>
        
            int main(int argc, char* argv[]) {
                printf("Hello zlib test...\\n");
                return 0;
            }
            """
        tmp_dir = tempfile.mkdtemp(prefix='helloztest_')
        bin_file_name = os.path.join(tmp_dir, 'helloz')
        file_name = bin_file_name + '.c'

        with open(file_name, 'w') as fp:
            fp.write(c_code)

        # compile and link code
        compiler = distutils.ccompiler.new_compiler()
        assert isinstance(compiler, distutils.ccompiler.CCompiler)
        distutils.sysconfig.customize_compiler(compiler)

        libraries = ['z']
        ret_val = True
        try:
            if verbose:
                print('checkZlib: compile and link')
            compiler.link_executable(
                compiler.compile([file_name]), bin_file_name, libraries=libraries, )
        except CompileError as e:
            if verbose:
                print('checkZlib: compile error in %s, zlib not available' % file_name)
            ret_val = False
        except LinkError as e:
            if verbose:
                print('checkZlib: link error in %s, zlib not available' % file_name)
            ret_val = False
        except Exception as e:
            if verbose:
                print('checkZlib: unknown error in %s, zlib not available' % file_name)
                logging.exception(e)
            ret_val = False
    except Exception as e:
        ret_val = False

    return ret_val


#%%

def get_version_info(verbose=0):
    """ Extract version information from source code """

    GIT_REVISION = None
    try:
        if os.path.exists('src/version.h'):
            with open('src/version.h') as f:
                ln = f.readline()
                m = re.search('.* "(.*)"', ln)
                FULLVERSION = (m.group(1))
        else:
            FULLVERSION = '0.0'
    except Exception as E:
        FULLVERSION = '0.0'
    if verbose:
        print('get_version_info: %s' % FULLVERSION)
    return FULLVERSION, GIT_REVISION

try:
    from distutils.version import LooseVersion
    from distutils.spawn import find_executable

    def get_swig_executable(swig_minimum_version='3.0', verbose=0):
        """ Get SWIG executable """
        # stolen from https://github.com/FEniCS/ffc/blob/master/setup.py

        swig_executable = None
        swig_version = None
        swig_valid = False
        for executable in ["swig", "swig3.0"]:
            swig_executable = find_executable(executable)
            if swig_executable is not None:
                # Check that SWIG version is ok
                output = subprocess.check_output([swig_executable, "-version"]).decode('utf-8')
                swig_version = re.findall(r"SWIG Version ([0-9.]+)", output)[0]
                if LooseVersion(swig_version) >= LooseVersion(swig_minimum_version):
                    swig_valid = True
                    break
        if verbose:
            print("Found SWIG: %s (version %s)" % (swig_executable, swig_version))
        return swig_executable, swig_version, swig_valid
    swig_executable, swig_version, swig_valid = get_swig_executable()
    print('swig_version %s, swig_executable %s' % (swig_version, swig_executable) )
except:
    def get_swig_executable():
        return None, None, False
    swig_valid = False

#%% Trick to remove option for c++ code compilation
try:
    # see http://stackoverflow.com/questions/8106258/cc1plus-warning-command-line-option-wstrict-prototypes-is-valid-for-ada-c-o
    from setuptools.py31compat import get_config_vars

    (opt,) = get_config_vars('OPT')

    if not opt is None:
        opt = " ".join(flag for flag in opt.split()
                       if flag != '-Wstrict-prototypes')
        os.environ['OPT'] = opt
except:
    import setuptools
    print('old version of setuptools: %s' % setuptools.__version__)
    pass


#%% Test suite

""" Run a limited set of tests for the package """
class OATest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        # New setuptools don't need this anymore, thus the try block.
        try:
            self.test_args = []
            self.test_suite = True
        except AttributeError:
            pass

    def run_tests(self):
        print('## oapackage test: load package')
        # import here, cause outside the eggs aren't loaded
        import oapackage.tests
        import oapackage.oahelper
        import oapackage.scanf
        import oapackage.tests.test_oapackage
        print('## oapackage test: oalib version %s' % oapackage.version())
        print('## oapackage test: package compile options\n%s\n' % oapackage.oalib.compile_information())

        oapackage.tests.test_oapackage.miscunittest(verbose=1)
        errno = 0
        sys.exit(errno)


#%% Define sources of the package
oadev = 0
srcs = ['arraytools.cpp', 'arrayproperties.cpp', 'pareto.cpp', 'nonroot.cpp',
        'mathtools.cpp', 'oaoptions.cpp', 'tools.cpp', 'md5.cpp', 'strength.cpp', 'graphtools.cpp']
srcs = srcs + ['Deff.cpp', 'evenodd.cpp']
srcs = srcs + ['conference.cpp']


srcs = srcs + ['lmc.cpp', 'extend.cpp']  # code used for extension
srcs = ['src/' + ff for ff in srcs]
if os.path.exists('dev/oadevelop.cpp'):
    oadev = 1
    print('Building development code')
    srcs = ['dev/oadevelop.cpp'] + srcs


sources = srcs + ['src/bitarray/bit_array.cpp']
oaheaders = [ cppfile.replace('.cpp', '.h') for cppfile in srcs ] +[os.path.join('src', 'version.h')]


for nauty_file in 'nauty.c nautinv.c nautil.c naurng.c naugraph.c schreier.c naugroup.c'.split(' '):
    sources += [os.path.join('src', 'nauty',nauty_file)]

nautyheaders = [os.path.join('src', 'nauty', headerfile) for headerfile in ['gtools.h', 'naugroup.h','nautinv.h',  'naurng.h', 'naugraph.h', 'nausparse.h','nautil.h',  'nauty.h', 'schreier.h']]
sources=sources

swig_opts = []
compile_options = []

sources = ['oalib.i'] + sorted(sources)
if oadev:
    swig_opts += ['-modern', '-c++', '-w503,401,362,509,389',
                  '-Isrc/', '-Idev/']
    compile_options += ['-DSWIGCODE', '-DFULLPACKAGE', '-DOADEV', '-Idev/']
    swig_opts += ['-DSWIGCODE', '-DFULLPACKAGE', '-DOADEV']
else:
    swig_opts += ['-modern', '-c++',
                  '-w503,401,362,302,389,446,509,305', '-Isrc/']
    compile_options += ['-DSWIGCODE', '-DFULLPACKAGE']
    swig_opts += ['-DSWIGCODE', '-DFULLPACKAGE']

# add nauty files
swig_opts += ['-Isrc/nauty/']
compile_options += ['-Isrc/nauty/']


if platform.system() == 'Windows':
    compile_options += ['-DWIN32', '-D_WIN32']
    swig_opts += ['-DWIN32', '-D_WIN32']

rtd = os.environ.get('READTHEDOCS', False)
print('Readthedocs environment: %s' % (rtd,))

if 'VSC_SCRATCH' in os.environ.keys():
    # we are running on the VSC cluster
    zlibdir = os.environ.get('EBROOTZLIB')  # SOFTROOTZLIB

    libraries = ['z']
    library_dirs = [zlibdir + '/lib']
    include_dirs = ['.', 'src', npinclude, zlibdir + '/include']
else:
    libraries = []
    library_dirs = []
    include_dirs = ['.', 'src', npinclude]

oalib_module = Extension('_oalib', sources=sources,
                         include_dirs=include_dirs, library_dirs=library_dirs, libraries=libraries, swig_opts=swig_opts
                         )

compile_options += ['-DNOOMP']
swig_opts += ['-DNOOMP']

oalib_module.extra_compile_args = compile_options

if checkZlib(verbose=1):
    if platform.system() == 'Windows':
        pass
    else:
        zlibflag = '-DUSEZLIB'
        oalib_module.extra_compile_args += [zlibflag]
        swig_opts += [zlibflag]
        oalib_module.extra_link_args += ['-lz']
else:
    zlibflag = '-DNOZLIB'
    oalib_module.extra_compile_args += [zlibflag]
    swig_opts += [zlibflag]

if os.name == 'nt':
    oalib_module.extra_compile_args += []
else:
    oalib_module.extra_compile_args += ['-O3', '-Wno-unknown-pragmas', '-Wno-sign-compare',
                                        '-Wno-return-type', '-Wno-unused-variable', '-Wno-unused-result', '-fPIC']
    oalib_module.extra_compile_args += ['-Wno-date-time', ]

if platform.node() == 'marmot' or platform.node() == 'goffer' or platform.node() == 'woelmuis':
    # openmp version of code
    oalib_module.extra_compile_args += ['-fopenmp', '-DDOOPENMP']
    oalib_module.extra_link_args += ['-fopenmp']

print('find_packages: %s' % find_packages())

data_files = []
scripts = ['misc/scripts/example_oapackage_python.py']
packages = find_packages()

# fix from:
# http://stackoverflow.com/questions/12491328/python-distutils-not-include-the-swig-generated-module

if rtd and 0:
    ext_modules = [] # do not build on RTD, this generates a time-out error  
    swigcmd = '%s -python -modern -c++ -w503,401,362,302,389,446,509,305 -Isrc/ -DSWIGCODE -DFULLPACKAGE -Isrc/nauty/ -DWIN32 -D_WIN32 -DNOOMP -DNOZLIB -o oalib_wrap.cpp oalib.i' % swig_executable
    print('RTD: run swig command: %s' % (swigcmd,))
    output = subprocess.check_output(swigcmd.split(' '))
    print('swig output:')
    print(output)
else:
    if not swig_valid:
        raise Exception('could not find a recent version if SWIG')

    ext_modules = [oalib_module]

# see: http://stackoverflow.com/questions/12491328/python-distutils-not-include-the-swig-generated-module
class CustomBuild(distutils_build):

    def run(self):
        self.run_command('build_ext')
        distutils_build.run(self)


class CustomInstall(setuptools_install):

    def run(self):
        self.run_command('build_ext')
        setuptools_install.run(self)


class BuildExtSwig3(setuptools.command.build_ext.build_ext):
    def find_swig(self):
        swig_executable, _, _ = get_swig_executable()
        return swig_executable
    
def readme():
    with open('README.md') as f:
        return f.read()


long_description = readme()

version = get_version_info()[0]
print('OApackage: version %s' % version)

if is_python3:
    python27_requirements = []
else:
    python27_requirements = ['mock; python_version <"3.0"', 'backports.functools_lru_cache;python_version<"2.9"']
    
setup(name='OApackage',
      cmdclass={'test': OATest, 'install': CustomInstall, 'build': CustomBuild, 'build_ext': BuildExtSwig3},
      version=version,
      author="Pieter Eendebak",
      description="Package to generate and analyse orthogonal arrays and optimal designs",
      long_description=long_description,
      long_description_content_type='text/markdown',
      author_email='pieter.eendebak@gmail.com',
      license="BSD",
      url='http://www.pietereendebak.nl/oapackage/index.html',
      keywords=["orthogonal arrays, design of experiments, conference designs, isomorphism testing"],
      ext_modules=ext_modules,
      py_modules=['oalib'],
      packages=packages,
      data_files=data_files,
      scripts=scripts,
      tests_require=['numpy', 'nose>=1.3', 'coverage>=4.0', 'mock' , 'python-dateutil']+python27_requirements,
      zip_safe=False,
      install_requires=['numpy>=1.13', 'python-dateutil'] + python27_requirements,
      extras_require={
          'GUI':  ["qtpy", 'matplotlib'],
          'documentation': ['sphinx']
      },
      requires=['numpy', 'matplotlib'],
      classifiers=['Development Status :: 4 - Beta', 'Intended Audience :: Science/Research',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.4',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6',
                   'Programming Language :: Python :: 3.7',
                   'License :: OSI Approved :: BSD License'
                   ]
      )



""" Orthogonal Array package

The Orthogonal Array package is a package to generate and analyse orthogonal
arrays, optimal designs and conference matrices. For more information see

http://github.com/eendebakpt/oapackage

"""

import oalib
import oapackage.Doptim
from . import conference
from oalib import *
from oalib import arraydata_t, array_link, exampleArray, ParetoDoubleLong, reduceOAnauty, arraylink2arraydata, reduceGraphNauty, transformGraphMatrix

oapackage.oalib.setloglevel(oapackage.oalib.SYSTEM)
oapackage.oalib.log_print(-oapackage.oalib.SYSTEM, '')
__version__ = oapackage.oalib.version()

from . oahelper import *
from . Doptim import *
from . import scanf


__description__ = "Orthogonal Array package"
__uri__ = "http://www.pietereendebak.nl/oapackage/index.html"
#__doc__ = __description__ + " <" + __uri__ + ">"
